import { defineComponent } from 'vue';
import * as common from 'schema-based-json-editor';
import { Select2 } from "select2-vue-component";
import { Icon } from './icon';
import { Editor } from './editor';
import { Optional } from './optional';
import { Description } from './description';
import { arrayEditorTemplateHtml } from './variables';
export var ArrayEditor = defineComponent({
    props: {
        schema: {
            type: Object,
            required: true,
        },
        initialValue: Array,
        title: [String, Number],
        getReference: {
            type: Function,
            required: true,
        },
        theme: {
            type: Object,
            required: true,
        },
        icon: {
            type: Object,
            requried: true,
        },
        locale: {
            type: Object,
            required: true,
        },
        readonly: Boolean,
        required: Boolean,
        hasDeleteButton: {
            type: Boolean,
            required: true,
        },
        dragula: Function,
        md: Object,
        hljs: Object,
        forceHttps: Boolean,
        disableCollapse: Boolean,
        noSelect2: Boolean,
        minItemCountIfNeedFilter: Number,
        monacoEditor: Object,
    },
    data: function () {
        return {
            renderSwitch: 1,
            collapsed: false,
            value: [],
            errorMessage: '',
            buttonGroupStyleString: common.buttonGroupStyleString,
            filter: '',
            invalidIndexes: [],
            drak: null,
        };
    },
    computed: {
        filteredValues: function () {
            var _this = this;
            return this.getValue.map(function (p, i) { return ({ p: p, i: i }); })
                .filter(function (_a) {
                var p = _a.p, i = _a.i;
                return common.filterArray(p, i, _this.schema.items, _this.filter);
            });
        },
        getValue: function () {
            if (this.value !== undefined && !this.collapsed) {
                return this.value;
            }
            return [];
        },
        isReadOnly: function () {
            return this.readonly || this.schema.readonly;
        },
        hasDeleteButtonFunction: function () {
            return this.hasDeleteButton && !this.isReadOnly;
        },
        hasAddButton: function () {
            return !this.isReadOnly && this.value !== undefined && !this.schema.enum;
        },
        titleToShow: function () {
            return common.getTitle(this.title, this.schema.title);
        },
        showFilter: function () {
            var minItemCountIfNeedFilter = typeof this.minItemCountIfNeedFilter === 'number' ? this.minItemCountIfNeedFilter : common.minItemCountIfNeedFilter;
            return this.getValue.length >= minItemCountIfNeedFilter;
        },
        className: function () {
            var rowClass = this.errorMessage ? this.theme.errorRow : this.theme.row;
            return this.schema.className ? rowClass + ' ' + this.schema.className : rowClass;
        },
        options: function () {
            return common.getOptions(this.schema);
        },
    },
    beforeMount: function () {
        this.collapsed = this.schema.collapsed;
        this.value = common.getDefaultValue(this.required, this.schema, this.initialValue);
        this.validate();
        this.$emit('update-value', { value: this.value, isValid: !this.errorMessage });
    },
    beforeUnmount: function () {
        if (this.drak) {
            this.drak.destroy();
        }
    },
    mounted: function () {
        var _this = this;
        if (this.dragula) {
            var container = common.findContainer(this.$el.childNodes);
            if (container) {
                this.drak = this.dragula([container]);
                this.drak.on('drop', function (el, target, source, sibling) {
                    if (_this.value) {
                        common.switchItem(_this.value, el, sibling);
                        _this.renderSwitch = -_this.renderSwitch;
                        _this.$emit('update-value', { value: _this.value, isValid: !_this.errorMessage && _this.invalidIndexes.length === 0 });
                    }
                });
            }
        }
    },
    methods: {
        collapseOrExpand: function () {
            this.collapsed = !this.collapsed;
        },
        toggleOptional: function () {
            this.value = common.toggleOptional(this.value, this.schema, this.initialValue);
            this.validate();
            this.$emit('update-value', { value: this.value, isValid: !this.errorMessage && this.invalidIndexes.length === 0 });
        },
        addItem: function () {
            this.value.push(common.getDefaultValue(true, this.schema.items, undefined));
            this.$emit('update-value', { value: this.value, isValid: !this.errorMessage && this.invalidIndexes.length === 0 });
        },
        onDeleteFunction: function (i) {
            this.value.splice(i, 1);
            this.renderSwitch = -this.renderSwitch;
            this.validate();
            this.$emit('update-value', { value: this.value, isValid: !this.errorMessage && this.invalidIndexes.length === 0 });
        },
        onChange: function (i, _a) {
            var value = _a.value, isValid = _a.isValid;
            this.value[i] = value;
            this.validate();
            common.recordInvalidIndexesOfArray(this.invalidIndexes, isValid, i);
            this.$emit('update-value', { value: this.value, isValid: !this.errorMessage && this.invalidIndexes.length === 0 });
        },
        onFilterChange: function (e) {
            this.filter = e.target.value;
        },
        isChecked: function (value) {
            return this.value && this.value.indexOf(value) !== -1;
        },
        onChangeCheckbox: function (value) {
            if (this.value) {
                var index = this.value.indexOf(value);
                if (index !== -1) {
                    this.value.splice(index, 1);
                }
                else {
                    this.value.push(value);
                }
                this.validate();
                this.$emit('update-value', { value: this.value, isValid: !this.errorMessage && this.invalidIndexes.length === 0 });
            }
        },
        onChangeSelect2: function (value) {
            this.value = value;
            this.validate();
            this.$emit('update-value', { value: this.value, isValid: !this.errorMessage && this.invalidIndexes.length === 0 });
        },
        validate: function () {
            this.errorMessage = common.getErrorMessageOfArray(this.value, this.schema, this.locale);
        },
    },
    components: {
        icon: Icon,
        optional: Optional,
        description: Description,
        editor: Editor,
        select2: Select2,
    },
    render: arrayEditorTemplateHtml,
});
