import { defineComponent } from 'vue';
import * as common from 'schema-based-json-editor';
import { Select2 } from "select2-vue-component";
import { Icon } from './icon';
import { Optional } from './optional';
import { Description } from './description';
import { Editor } from './editor';
import { objectEditorTemplateHtml } from './variables';
export var ObjectEditor = defineComponent({
    render: objectEditorTemplateHtml,
    components: {
        icon: Icon,
        optional: Optional,
        description: Description,
        editor: Editor,
        select2: Select2,
    },
    props: {
        schema: {
            type: Object,
            required: true,
        },
        initialValue: Object,
        title: [String, Number],
        getReference: {
            type: Function,
            required: true,
        },
        theme: {
            type: Object,
            required: true,
        },
        icon: {
            type: Object,
            required: true,
        },
        locale: {
            type: Object,
            required: true,
        },
        readonly: Boolean,
        required: Boolean,
        hasDeleteButton: {
            type: Boolean,
            required: true,
        },
        dragula: Function,
        md: Object,
        hljs: Object,
        forceHttps: Boolean,
        disableCollapse: Boolean,
        noSelect2: Boolean,
        minItemCountIfNeedFilter: Number,
        monacoEditor: Object,
    },
    data: function () {
        return {
            collapsed: false,
            value: {},
            buttonGroupStyle: common.buttonGroupStyleString,
            errorMessage: '',
            filter: '',
            invalidProperties: [],
            properties: [],
            watchedProperties: [],
        };
    },
    beforeMount: function () {
        this.collapsed = this.schema.collapsed;
        this.value = common.getDefaultValue(this.required, this.schema, this.initialValue);
        this.validate();
        if (this.value !== undefined) {
            var _loop_1 = function (property) {
                if (this_1.schema.properties.hasOwnProperty(property)) {
                    var schema = this_1.schema.properties[property];
                    var propertyName = schema.propertyName || property;
                    if (this_1.isRequired(property) !== false) {
                        var required = this_1.schema.required && this_1.schema.required.some(function (r) { return r === property; });
                        this_1.value[propertyName] = common.getDefaultValue(required, schema, this_1.value[propertyName]);
                    }
                    this_1.properties.push({
                        propertyName: propertyName,
                        property: property,
                        schema: schema
                    });
                }
            };
            var this_1 = this;
            for (var property in this.schema.properties) {
                _loop_1(property);
            }
            this.properties.sort(common.compare);
        }
        for (var property in this.schema.properties) {
            var schema = this.schema.properties[property];
            if (schema && schema.requiredWhen && !this.watchedProperties.includes(schema.requiredWhen[0])) {
                this.watchedProperties.push(schema.requiredWhen[0]);
            }
        }
        this.$emit('update-value', { value: this.value, isValid: true });
    },
    computed: {
        filteredProperties: function () {
            var _this = this;
            return this.properties.filter(function (p) { return common.filterObject(p, _this.filter); });
        },
        isReadOnly: function () {
            return this.readonly || this.schema.readonly;
        },
        hasDeleteButtonFunction: function () {
            return this.hasDeleteButton && !this.isReadOnly;
        },
        titleToShow: function () {
            if (this.hasDeleteButton) {
                return common.getTitle(common.findTitle(this.value, this.properties), this.title, this.schema.title);
            }
            return common.getTitle(this.title, this.schema.title);
        },
        showFilter: function () {
            var _this = this;
            var propertyCount = this.properties.filter(function (p) { return _this.isRequired(p.property) !== false; }).length;
            var minItemCountIfNeedFilter = typeof this.minItemCountIfNeedFilter === 'number' ? this.minItemCountIfNeedFilter : common.minItemCountIfNeedFilter;
            return propertyCount >= minItemCountIfNeedFilter;
        },
        className: function () {
            var rowClass = this.errorMessage ? this.theme.errorRow : this.theme.row;
            return this.schema.className ? rowClass + ' ' + this.schema.className : rowClass;
        },
    },
    methods: {
        isRequired: function (property) {
            return common.isRequired(this.schema.required, this.value, this.schema, property);
        },
        collapseOrExpand: function () {
            this.collapsed = !this.collapsed;
        },
        toggleOptional: function () {
            this.value = common.toggleOptional(this.value, this.schema, this.initialValue);
            this.validate();
            this.$emit('update-value', { value: this.value, isValid: this.invalidProperties.length === 0 });
        },
        onChange: function (property, _a) {
            var value = _a.value, isValid = _a.isValid;
            this.value[property] = value;
            for (var p in this.schema.properties) {
                if (this.isRequired(p) === false) {
                    this.value[p] = undefined;
                }
            }
            this.validate();
            if (this.watchedProperties.some(function (p) { return p === property; })) {
                this.$forceUpdate();
            }
            common.recordInvalidPropertiesOfObject(this.invalidProperties, isValid, property);
            this.$emit('update-value', { value: this.value, isValid: this.invalidProperties.length === 0 });
        },
        onFilterChange: function (e) {
            this.filter = e.target.value;
        },
        validate: function () {
            this.errorMessage = common.getErrorMessageOfObject(this.value, this.schema, this.locale);
        },
    }
});
